{$A8,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O-,P+,Q+,R+,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$MINSTACKSIZE $00004000}
{$MAXSTACKSIZE $00100000}
{$IMAGEBASE $00400000}
{$APPTYPE GUI}
unit Draughts;

interface
Uses
  Classes;

Type
  TMove=Record
    dX, dY:Integer;
  End;

Const
  ctWhite=0;
  ctBlack=1;
  ctNone=255;
  
  sdWhite=0;
  sdBlack=1;
  kdSimple=0;
  kdKing=1;

  MoveNW=1; {}
  MoveNE=2; {}
  MoveSW=3; {}
  MoveSE=4; {}

  DraughtMoves:Array[1..4] Of TMove=((dX:-1; dY:1), (dX:1; dY:1), (dX:-1; dY:-1), (dX:1; dY:-1));

Type
  TPossibleMoves=Set Of MoveNW..MoveSE;
  PCellPosition=^TCellPosition;
  TCellPosition=Record
    X, Y:Integer;
  End;

  TDraught=Class(TObject)
  Private
    _Side:Integer; {: /}
    _Kind:Integer; {: /}
    PossibleMoves:TPossibleMoves; { }

    Procedure SetSide(Value:Integer);
    Procedure SetKind(Value:Integer);
  Public
    Constructor Create(sd:Integer);
    Destructor Destroy; Override;

    Property Side:Integer read _Side write SetSide;
    Property Kind:Integer read _Kind write SetKind;
  End;

  TBoardCell=Record
    CellType:Integer;
    Draught:TDraught;
  End;
  TChessBoard=Class(TObject)

  Private
    Cells:Array[0..9, 0..9] Of TBoardCell;

  Public

    Constructor Create;
    Destructor Destroy; Override;

    Function AddDraught(x, y:Integer; Side:Integer; Kind:Integer=kdSimple):TDraught;
    Procedure DeleteDraught(x, y:Integer);
    Function GetDraught(x, y:Integer):TDraught;
    Function SetDraught(x, y:Integer; Const Draught:TDraught):Boolean;
    Procedure Clear;
    Procedure Respawn;
    Function MoveDraught(FromX, FromY, ToX, ToY:Integer):Boolean;
    Procedure FindMoves(x, y:Integer; Var Moves, Strikes, Strickens:TList);
    Function FindStrikingDraughts(Side:Integer):TList;
    Function CountDraughts(Side:Integer):Integer;
    Function AbleToMove(Side:Integer):Boolean;
  End;

  Procedure ClearMoves(Var Moves:TList);
  Function CheckPosition(x, y:Integer; Const Positions:TList):Integer;

implementation

Procedure ClearMoves(Var Moves:TList);
Var
  i:Integer;
  CP:PCellPosition;
Begin
  If Moves=nil Then Exit;
  For i:=0 To Moves.Count-1 Do
  Begin
    CP:=Moves.Items[i];
    Dispose(CP);
  End;
  Moves.Free;
  Moves:=nil;
End;

Function CheckPosition(x, y:Integer; Const Positions:TList):Integer;
Var
  i:Integer;
  CP:PCellPosition;
Begin
  Result:=-1;
  If Positions=nil Then Exit;
  For i:=0 To Positions.Count-1 Do
  Begin
    CP:=Positions.Items[i];
    If ((CP^.X=x) And (CP^.Y=y)) Then
    Begin
      Result:=i;
      Exit;
    End;
  End;
End;


Constructor TDraught.Create(sd:Integer);
Begin
  Inherited Create;

  Side:=sd;
  Kind:=kdSimple;
End;

Destructor TDraught.Destroy;
Begin

  Inherited Destroy;
End;

Procedure TDraught.SetSide(Value:Integer);
Begin
  If Value=sdWhite Then
  Begin
    _Side:=sdWhite;
    PossibleMoves:=[MoveNW, MoveNE]
  End
  Else Begin
    _Side:=sdBlack;
    PossibleMoves:=[MoveSW, MoveSE];
  End;
  If Kind=kdKing Then PossibleMoves:=[MoveNW, MoveNE, MoveSW, MoveSE];
End;

Procedure TDraught.SetKind(Value:Integer);
Begin
  If Value=kdSimple Then
    _Kind:=kdSimple
  Else
    _Kind:=kdKing;
  SetSide(_Side);
End;


Constructor TChessBoard.Create;
Var
  x, y:Integer;
Begin
  Inherited Create;

  For y:=0 To 9 Do
  Begin
    For x:=0 To 9 Do
    Begin
      If ((x=0) Or (y=0) Or (x=9) Or (y=9)) Then
        Cells[y, x].CellType:=ctNone
      Else
        If ((Odd(x) And Odd(y)) Or (Not(Odd(x)) And Not(Odd(y)))) Then
          Cells[y, x].CellType:=ctBlack
        Else
          Cells[y, x].CellType:=ctWhite;
      Cells[y, x].Draught:=nil;
    End;
  End;
End;

Destructor TChessBoard.Destroy;
Begin
  Clear;
  Inherited Destroy;
End;

Function TChessBoard.AddDraught(x, y:Integer; Side:Integer; Kind:Integer=kdSimple):TDraught;
Begin
  Result:=nil;

  If ((Cells[y, x].CellType=ctNone) Or (Cells[y, x].Draught<>nil)) Then Exit;

  Result:=TDraught.Create(Side);
  Result.Kind:=Kind;
  Cells[y, x].Draught:=Result;
End;

Procedure TChessBoard.DeleteDraught(x, y:Integer);
Begin
  If Cells[y, x].Draught<>nil Then Cells[y, x].Draught.Free;
  Cells[y, x].Draught:=nil;
End;

Function TChessBoard.GetDraught(x, y:Integer):TDraught;
Begin
  Result:=Cells[y, x].Draught;
End;

Function TChessBoard.SetDraught(x, y:Integer; Const Draught:TDraught):Boolean;
Begin
  Result:=True;
  Cells[y, x].Draught:=Draught;
  If Draught=nil Then Exit;
  If (((Draught.Side=sdWhite) And (y=8)) Or ((Draught.Side=sdBlack) And (y=1))) Then
    Draught.Kind:=kdKing;
End;

Procedure TChessBoard.Clear;
Var
  x, y:Integer;
Begin
  For y:=0 To 9 Do
    For x:=0 To 9 Do
      DeleteDraught(x, y);
End;

Procedure TChessBoard.Respawn;
Var
  x:Integer;
Begin
  Clear;
//
  For x:=1 To 8 Do
  Begin
    If Odd(x) Then
      AddDraught(x, 2, sdWhite)
    Else Begin
      AddDraught(x, 1, sdWhite);
      AddDraught(x, 3, sdWhite);
    End
  End;
//
  For x:=1 To 8 Do
  Begin
    If Not(Odd(x)) Then
      AddDraught(x, 7, sdBlack)
    Else Begin
      AddDraught(x, 8, sdBlack);
      AddDraught(x, 6, sdBlack);
    End
  End;
End;

Function TChessBoard.MoveDraught(FromX, FromY, ToX, ToY:Integer):Boolean;
Begin
  Result:=False;
  If ((Cells[FromY, FromX].CellType=ctNone) Or (Cells[ToY, ToX].CellType=ctNone)) Then Exit;
  If ((GetDraught(FromX, FromY)=nil) Or (GetDraught(ToX, ToY)<>nil)) Then Exit;

  SetDraught(ToX, ToY, GetDraught(FromX, FromY));
  SetDraught(FromX, FromY, nil);
  Result:=True;
End;

Procedure TChessBoard.FindMoves(x, y:Integer; Var Moves, Strikes, Strickens:TList);
Var
  i:Integer;
  dX, dY, dL:Integer;
  P:PCellPosition;
  MoveOver:Boolean;
Begin
  Moves:=nil;
  Strikes:=nil;
  Strickens:=nil;
  If Cells[y, x].Draught=nil Then Exit;

  Moves:=TList.Create;
  Strikes:=TList.Create;
  Strickens:=TList.Create;

  For i:=1 To 4 Do
  Begin
    MoveOver:=False;
    dX:=DraughtMoves[i].dX;
    dY:=DraughtMoves[i].dY;
    dL:=0;

    Repeat
      Inc(dL);
      If Cells[y+dY*dL, x+dX*dL].CellType<>ctNone Then  //     
      Begin
        If Cells[y+dY*dL, x+dX*dL].Draught=nil Then  //  ,   ...
        Begin
         //...    ,       
          If (i In Cells[y, x].Draught.PossibleMoves) Then
          Begin
            New(P);
            P^.X:=x+dX*dL; P^.Y:=y+dY*dL;
            Moves.Add(P);
            If Cells[y, x].Draught.Kind=kdSimple Then MoveOver:=True;  //   ,   
          End
          Else  //       -  
            MoveOver:=True;
        End
        Else Begin  //    ,   
          If Cells[y+dY*dL, x+dX*dL].Draught.Side<>Cells[y, x].Draught.Side Then
          Begin   //   ,    
            Inc(dL);  //  
            If ((Cells[y+dY*dL, x+dX*dL].CellType<>ctNone) And (Cells[y+dY*dL, x+dX*dL].Draught=nil)) Then  //     
            Begin // 
             //  
              New(P);
              P^.X:=x+dX*(dL-1); P^.Y:=y+dY*(dL-1);
              Strickens.Add(P);
             // 
              New(P);
              P^.X:=x+dX*dL; P^.Y:=y+dY*dL;
              Strikes.Add(P);
            End;
          End;
          MoveOver:=True;
        End;
      End
      Else  //  -  
        MoveOver:=True;
    Until MoveOver;
  End;
End;

Function TChessBoard.FindStrikingDraughts(Side:Integer):TList;
Var
  x, y:Integer;
  Moves, Strikes, Strickens:TList;
  CP:PCellPosition;
Begin
  Result:=TList.Create;

  For y:=1 To 8 Do
    For x:=1 To 8 Do
      If GetDraught(x, y)<>nil Then
        If GetDraught(x, y).Side=Side Then
        Begin
          FindMoves(x, y, Moves, Strikes, Strickens);
          If Strikes.Count>0 Then
          Begin
            New(CP);
            CP^.X:=x; CP^.Y:=y;
            Result.Add(CP);
          End;
          ClearMoves(Moves);
          ClearMoves(Strikes);
          ClearMoves(Strickens);
        End;
End;

Function TChessBoard.CountDraughts(Side:Integer):Integer;
Var
  x, y:Integer;
Begin
  Result:=0;
  For y:=1 To 8 Do
    For x:=1 To 8 Do
      If GetDraught(x, y)<>nil Then
        If GetDraught(x, y).Side=Side Then Inc(Result);
End;

Function TChessBoard.AbleToMove(Side:Integer):Boolean;
Var
  x, y:Integer;
  Moves, Strikes, Strickens:TList;
Begin
  Result:=False;
  For y:=1 To 8 Do
    For x:=1 To 8 Do
      If GetDraught(x, y)<>nil Then
        If GetDraught(x, y).Side=Side Then
        Begin
          FindMoves(x, y, Moves, Strikes, Strickens);
          If ((Moves.Count>0) Or (Strikes.Count>0)) Then Result:=True;
          ClearMoves(Moves);
          ClearMoves(Strikes);
          ClearMoves(Strickens);
          If Result Then Exit;
        End;
End;

end.
