{$A8,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O-,P+,Q+,R+,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$MINSTACKSIZE $00004000}
{$MAXSTACKSIZE $00100000}
{$IMAGEBASE $00400000}
{$APPTYPE GUI}
unit VChessBoard;

interface
Uses
  Types, Classes, ExtCtrls,
  Draughts;

Type
  TVisualChessBoard=Class(TImage)

  Private
    Procedure DrawDraught(x, y:Integer; Side:Integer; Kind:Integer);
    Procedure DrawMove(x, y:Integer);
    Procedure DrawStrike(x, y:Integer);

  Public
    Board:TChessBoard;

    Constructor Create(AOwner:TComponent); Override;
    Destructor Destroy; Override;

    Procedure Repaint; Override;

    Function GetCellRectByNumber(x, y:Integer):TRect;
    Procedure GetCellNumberByCoord(xCoord, yCoord:Integer; Var x, y:Integer);

    Procedure DrawMoves(Const Moves, Strikes:TList);
    Procedure DrawMovesByNumber(x, y:Integer);
  End;

implementation
Uses
  Graphics;

Constructor TVisualChessBoard.Create(AOwner:TComponent);
Begin
  Inherited Create(AOwner);

  Board:=TChessBoard.Create;
//  Repaint;
End;

Destructor TVisualChessBoard.Destroy;
Begin
  Board.Free;
  
  Inherited Destroy;
End;

Procedure TVisualChessBoard.Repaint;
Var
  x, y:Integer;
Begin
  Inherited Repaint;

{ }
  Canvas.Brush.Style:=bsSolid;
  For y:=1 To 8 Do
  Begin
    For x:=1 To 8 Do
    Begin
      If ((Odd(x) And Odd(y)) Or (Not(Odd(x)) And Not(Odd(y)))) Then
        Canvas.Brush.Color:=clBlack
      Else
        Canvas.Brush.Color:=clWhite;

      Canvas.FillRect(GetCellRectByNumber(x, y));
    End;
  End;

  Canvas.Pen.Color:=clBlack;
  Canvas.Pen.Style:=psSolid;
  Canvas.Pen.Width:=1;
  Canvas.Brush.Style:=bsClear;
//  Canvas.Rectangle(0, (Height mod 8), Width-(Width mod 8), Height);
  Canvas.Rectangle(0, 0, Width, Height);

{ }
  If Board=nil Then Exit;

  For y:=1 To 8 Do
  Begin
    For x:=1 To 8 Do
    Begin
      If Board.GetDraught(x, y)<>nil Then
        DrawDraught(x, y, Board.GetDraught(x, y).Side, Board.GetDraught(x, y).Kind);
    End;
  End;
End;

Function TVisualChessBoard.GetCellRectByNumber(x, y:Integer):TRect;
Begin
  Result.Left:=Trunc((x-1)*(Width/8));
  Result.Right:=Trunc(x*(Width/8));
  Result.Top:=Height-Trunc(y*(Height/8));
  Result.Bottom:=Height-Trunc((y-1)*(Height/8));
End;

Procedure TVisualChessBoard.GetCellNumberByCoord(xCoord, yCoord:Integer; Var x, y:Integer);
Begin
  x:=1+Trunc(xCoord/(Width/8));
  y:=8-Trunc(yCoord/(Height/8));
End;

Procedure TVisualChessBoard.DrawDraught(x, y:Integer; Side:Integer; Kind:Integer);
Const
  dR=5;
Var
  R, tR:TRect;
Begin
  Canvas.Pen.Color:=clSilver;
  Canvas.Pen.Style:=psSolid;
  Canvas.Pen.Width:=3;
  Canvas.Brush.Style:=bsSolid;

  If Side=sdWhite Then Canvas.Brush.Color:=clWhite Else Canvas.Brush.Color:=clBlack;
  R:=GetCellRectByNumber(x, y);
  Canvas.Ellipse(R.Left+dR, R.Top+dR, R.Right-dR, R.Bottom-dR);

  If Kind=kdKing Then Begin
    tR:=R;
    R.Left:=tR.Left+((tR.Right-tR.Left) div 4);
    R.Right:=tR.Right-((tR.Right-tR.Left) div 4);
    R.Top:=tR.Top+((tR.Bottom-tR.Top) div 4);
    R.Bottom:=tR.Bottom-((tR.Bottom-tR.Top) div 4);
    Canvas.Ellipse(R);
  End;
End;

Procedure TVisualChessBoard.DrawMove(x, y:Integer);
Const
  dR=5;
Var
  R:TRect;
Begin
  Canvas.Pen.Color:=clLime;
  Canvas.Pen.Style:=psSolid;
  Canvas.Pen.Width:=1;
  Canvas.Brush.Color:=clLime;
  Canvas.Brush.Style:=bsDiagCross;

  R:=GetCellRectByNumber(x, y);
  Canvas.Ellipse(R.Left+dR, R.Top+dR, R.Right-dR, R.Bottom-dR);
End;

Procedure TVisualChessBoard.DrawStrike(x, y:Integer);
Const
  dR=5;
Var
  R:TRect;
Begin
  Canvas.Pen.Color:=clRed;
  Canvas.Pen.Style:=psSolid;
  Canvas.Pen.Width:=1;
  Canvas.Brush.Color:=clRed;
  Canvas.Brush.Style:=bsDiagCross;

  R:=GetCellRectByNumber(x, y);
  Canvas.Ellipse(R.Left+dR, R.Top+dR, R.Right-dR, R.Bottom-dR);
End;

Procedure TVisualChessBoard.DrawMoves(Const Moves, Strikes:TList);
Var
  CP:PCellPosition;
  i:Integer;
Begin
  If Moves<>nil Then
    For i:=0 To Moves.Count-1 Do
    Begin
      CP:=Moves.Items[i];
      DrawMove(CP^.X, CP^.Y);
    End;
  If Strikes<>nil Then
    For i:=0 To Strikes.Count-1 Do
    Begin
      CP:=Strikes.Items[i];
      DrawStrike(CP^.X, CP^.Y);
    End;
End;

Procedure TVisualChessBoard.DrawMovesByNumber(x, y:Integer);
Var
  Moves, Strikes, Strickens:TList;
Begin
  Board.FindMoves(x, y, Moves, Strikes, Strickens);
  DrawMoves(Moves, Strikes);
  ClearMoves(Moves);
  ClearMoves(Strikes);
  ClearMoves(Strickens);
End;

end.
